#!/bin/bash

# For safety reasons
ulimit -v 8000000

# Give each call only 1h CPU-Time
ulimit -t 3600

# Check usage
if [ $# -ne 1 ]
then
	echo "Usage: ${0} DIRECTORY
	- DIRECTORY is a directory where the testfiles/inputfiles are in

	For each file in DIRECTORY the algorithms specified in this script will be executed.
	Results of the algorithms will be printed into the file '${RESULTFILE}'.
	Information about the output of the algorithms will be written into the file '${RCHECKFILE}'
	"
	exit
fi

# Prepare
DATUM=`date +"%Y%m%d_%H%M%S"`
INPUTDIR=$1

# Run for all input files
for INPUTFILE in ${INPUTDIR}/*
do
	### FILE INFO ###
	FILENAME=`basename ${INPUTFILE}`
	FILEPATH=`dirname ${INPUTFILE}`
	FILESIZE=(`stat -c"%s" ${INPUTFILE}`)
	timestamp=`date +%s`

	echo "======= ${INPUTFILE} ======="
	# ./sais.x ${INPUTFILE} suffix_array &> /dev/null
	./create_sa.x ${INPUTFILE} suffix_array &> /dev/null
	for run in {1..1}
	do
		time ./xkcd_repeats_CT1.x ${INPUTFILE} suffix_array 3 3
		time ./xkcd_repeats_CT2.x ${INPUTFILE} suffix_array 3 3
		time ./xkcd_repeats_CT3.x ${INPUTFILE} suffix_array 3 3
	done
	rm suffix_array

	echo "========================================"
done
