#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <stack>
#include "phi.hpp"

using namespace std;

uint64_t found_supermax = 0;
uint64_t found_nearsuper = 0;
uint64_t found_max = 0;

vector<int64_t> computeSingletons(const int64_t lb, const int64_t rb, const vector<int64_t>& lcpIndices)
{
	vector<int64_t> singletons;
	if(lcpIndices.size())
	{
		if(lb == lcpIndices[0]-1)
		{
			singletons.push_back(lb);
		}
		for(uint64_t i=1; i<lcpIndices.size(); ++i)
		{
			if(lcpIndices[i-1] == lcpIndices[i]-1)
			{
				singletons.push_back(lcpIndices[i-1]);
			}
		}
		if(lcpIndices[lcpIndices.size()-1] == rb)
		{
			singletons.push_back(lcpIndices[lcpIndices.size()-1]);
		}
	}
	return singletons;
}

struct lcp_interval
{
	int64_t lcp;
	int64_t lb;
	vector<int64_t> lcpIndices;
	lcp_interval(int64_t _lcp, int64_t _lb) : lcp(_lcp), lb(_lb) { }
	lcp_interval(int64_t _lcp, int64_t _lb, int64_t _indice) : lcp(_lcp), lb(_lb)
	{
		lcpIndices.push_back(_indice);
	}
};

void max_nearsuper_super_repeats(const vector<int32_t>& lcp, const vector<uint8_t>& bwt, const uint64_t largest_symbol, const int64_t l_min)
{
	uint64_t lastdiff = 0;
	vector<int64_t> penultimate(largest_symbol+1, -1);
	vector<int64_t> last(largest_symbol+1, -1);
	last[bwt[0]] = 0;
	stack<lcp_interval> s;
	s.push(lcp_interval(-1, 0));
	for(uint64_t k=1; k<lcp.size(); ++k)
	{
		uint64_t lb = k-1;
		while(lcp[k] < s.top().lcp)
		{
			lcp_interval cur = s.top(); s.pop();
			lb = cur.lb;
			uint64_t rb = k-1;
			if(cur.lcp >= l_min and lastdiff > lb)
			{
				uint64_t witness = 0;
				vector<int64_t> singletons = computeSingletons(lb, rb, cur.lcpIndices);
				for(auto j : singletons) {
					uint8_t c = bwt[j];
					if(j == last[c] and penultimate[c] < (int64_t)lb)
					{
						++witness;
					}
				}
				if(witness == rb-lb+1)
				{
					++found_supermax;
				}
				else if(witness)
				{
					++found_nearsuper;
				}
				else
				{
					++found_max;
				}
			}
		}
		if(lcp[k] > s.top().lcp)
		{
			s.push(lcp_interval(lcp[k], lb, k));
		}
		else
		{
			s.top().lcpIndices.push_back(k);
		}
		uint8_t c = bwt[k];
		penultimate[c] = last[c];
		last[c] = k;
		if(bwt[k-1] != c) {
			lastdiff = k;
		}
	}
}


int main(int argc, char **argv)
{
	// Check Parameters
	if(argc!=4)
	{
		cerr << "Program takes path of input, its suffix array (as 4-Byte Integer file) and lmin." << endl;
		return 1;
	}

	uint64_t n;
	vector<uint8_t> text;
	vector<uint32_t> sa;

	// Read Text, as 1-Byte-Char File
	{
		ifstream input(argv[1], std::ios::in|std::ios::binary|std::ios::ate);
		n = input.tellg();
		input.seekg(0);
		text.resize(n+1);
		input.read((char*)text.data(), n);
		// Append Sentinal
		text[n] = 0;
		++n;
	}

	// Read Suffix-Array, as 4-Byte-Integer File
	{
		ifstream input(argv[2], std::ios::in|std::ios::binary);
		sa.resize(n);
		input.read((char*)sa.data(), 4*n);
	}

	// Create LCP-Array with phi
	vector<int32_t> lcp(sa.size()+1, 0);
	PHI(text, sa, lcp);

	// Create BWT
	vector<uint8_t> bwt(sa.size());
	for(uint64_t i=0; i<bwt.size(); ++i)
	{
		if(sa[i])
		{
			bwt[i] = text[sa[i]-1];
		}
		else
		{
			bwt[i] = text[text.size()-1];
		}
	}
	uint64_t largest_symbol = 0;
	for(uint64_t i=0; i<text.size(); ++i)
	{
		if(largest_symbol < text[i])
		{
			largest_symbol = text[i];
		}
	}
	int64_t l_min = atoi(argv[3]);

	found_supermax = found_nearsuper = found_max = 0;
	max_nearsuper_super_repeats(lcp, bwt, largest_symbol, l_min);
	cout << found_supermax << "\tsupermaixmal repeats" << endl;
	cout << found_nearsuper << "\tnear supermaximal repeats"<< endl;
	cout << found_max << "\tmaximal repeats"<< endl;
	
	return 0;
}
